// krazy:skip
//begin strings from hplip-3.9.2/scan/sane/common.h to make them translatable
I18N_NOOP("None");
I18N_NOOP("MH");
I18N_NOOP("MR");
I18N_NOOP("MMR");
I18N_NOOP("JPEG");
I18N_NOOP("Auto");
I18N_NOOP("Flatbed");
I18N_NOOP("ADF");
I18N_NOOP("Advanced");
I18N_NOOP("Compression");
I18N_NOOP("Selects the scanner compression "\
          "method for faster scans, possibly at the expense of image quality.");
I18N_NOOP("JPEG compression factor");
I18N_NOOP("Sets the scanner JPEG compression "\
          "factor. Larger numbers mean better compression, " \
          "and smaller numbers mean better image quality.");
I18N_NOOP("Batch scan");
I18N_NOOP("Enables continuous scanning with automatic document feeder (ADF).");
I18N_NOOP("Duplex");
I18N_NOOP("Enables scanning on both sides of the page.");
I18N_NOOP("Geometry");
I18N_NOOP("Length measurement");
I18N_NOOP("Selects how the scanned image "\
          "length is measured and reported, which is impossible to know in advance for scrollfed scans.");
I18N_NOOP("Unknown");
I18N_NOOP("Unlimited");
I18N_NOOP("Approximate");
I18N_NOOP("Padded");
I18N_NOOP("Exact");
I18N_NOOP("???");
//end strings from hplip-3.9.2/scan/sane/common.h to make them translatable
